'' =================================================================================================
''
''   File....... jm_ws2812ss_demo.spin
''   Purpose.... WS2812 demonstration program (single-shot variant)
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 16 SEP 2013
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     ' 16x required for WS2812                              
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LEDS =  6                                                     ' LED tx pin
           
  
obj

  strip : "jm_ws2812ss"                                         ' WS2812 LED driver
  prng  : "jm_prng"


var

  long  local[40]                                               ' local array


dat

  Chakras               long    $06_00_00
                        long    $05_02_00
                        long    $04_04_00
                        long    $00_07_00
                        long    $00_00_08
                        long    $03_00_04


pub main | idx

  strip.start                                                   ' start led driver
  prng.start                                                    ' start randomizer
  prng.seed(cnt, 0, 0, 0, 0)
  pause(10)


  repeat
    strip.set_all($06_00_00, 0, 40)                             ' fill with color
    strip.execute(LEDS, 40, 0)                                  ' display
    pause(333)
    strip.set_all($00_07_00, 0, 40)
    strip.execute(LEDS, 40, 0)
    pause(333)
    strip.set_all($00_00_08, 0, 40)
    strip.execute(LEDS, 40, 0)
    pause(333)

    ' randomize and diplay local array

    repeat idx from 0 to 39
      local[idx] := Chakras[(prng.random >> 1) // 6]
    strip.execute(LEDS, 40, @local)
    pause(1000)

    ' clear and display local array

    strip.set_all($00_00_00, @local, 40)
    strip.execute(LEDS, 40, @local) 
    pause(333)

    ' fill demo

    strip.fill(0, 39, $03_00_04, @local)
    strip.execute(LEDS, 40, @local)
    pause(250)
    strip.fill(0, 31, $00_00_08, @local)
    strip.execute(LEDS, 32, @local)
    pause(250)
    strip.fill(0, 23, $00_07_00, @local)
    strip.execute(LEDS, 24, @local)
    pause(250)
    strip.fill(0, 15, $04_04_00, @local)
    strip.execute(LEDS, 16, @local)
    pause(250)
    strip.fill(0, 7, $06_00_00, @local)
    strip.execute(LEDS, 8, @local)
    pause(1000)
    
   
con

  ' Routines ported from C code by Phil Burgess (www.paintyourdragon.com)

 
pub wheel(pos)

'' Creates color from 0 to 255 position input
'' -- colors transition r->g->b back to r

  if (pos < 85)
    return strip.color(255-pos*3, pos*3, 0)
  elseif (pos < 170)
    pos -= 85
    return strip.color(0, 255-pos*3, pos*3)
  else
    pos -= 170
    return strip.color(pos*3, 0, 255-pos*3)


pub wheelx(pos, level)                                          ' update by JonnyMac

'' Creates color from 0 to 255 position input
'' -- colors transition r-g-b back to r
'' -- level is brightness, 0 to 255

  if (pos < 85)
    return strip.colorx(255-pos*3, pos*3, 0, level)
  elseif (pos < 170)
    pos -= 85
    return strip.colorx(0, 255-pos*3, pos*3, level)
  else
    pos -= 170
    return strip.colorx(pos*3, 0, 255-pos*3, level)

 
con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
 

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}